@@ -42,6 +42,7 @@ from rich.text import Text
 
 from cecli.commands import SwitchCoderSignal
 from cecli.helpers import coroutines
+from cecli.report import update_error_prefix
 
 from .dump import dump  # noqa: F401
 from .editor import pipe_editor
@@ -235,16 +236,10 @@ class AutoCompleter(Completer):
             yield from raw_completer(document, complete_event)
             return
 
-        if cmd not in self.command_completions:
-            candidates = self.commands.get_completions(cmd)
-            self.command_completions[cmd] = candidates
-        else:
-            candidates = self.command_completions[cmd]
+        candidates = self.commands.get_completions(cmd, partial)
 
         if candidates is None:
             return
-
-        candidates = [word for word in candidates if partial in word.lower()]
         for candidate in sorted(candidates):
             yield Completion(candidate, start_position=-len(words[-1]))
 
@@ -396,9 +391,9 @@ class InputOutput:
 
         self.fzf_available = shutil.which("fzf")
         if not self.fzf_available and self.verbose:
-            self.tool_warning(
-                "fzf not found, fuzzy finder features will be disabled. Install it for enhanced"
-                " file/history search."
+            print(
+                "Warning: fzf not found, fuzzy finder features will be disabled. Install it for"
+                " enhanced file/history search."
             )
 
         self.code_theme = code_theme
@@ -421,7 +416,7 @@ class InputOutput:
             try:
                 Path(self.input_history_file).parent.mkdir(parents=True, exist_ok=True)
             except (PermissionError, OSError) as e:
-                self.tool_warning(f"Could not create directory for input history: {e}")
+                print(f"Warning: Could not create directory for input history: {e}")
                 self.input_history_file = None
 
         if chat_history_file is not None:
@@ -490,11 +485,11 @@ class InputOutput:
                 self.console = Console()  # pretty console
             except Exception as err:
                 self.console = Console(force_terminal=False, no_color=True)
-                self.tool_error(f"Can't initialize prompt toolkit: {err}")  # non-pretty
+                print(f"Error: Can't initialize prompt toolkit: {err}")  # non-pretty
         else:
             self.console = Console(force_terminal=False, no_color=True)  # non-pretty
             if self.is_dumb_terminal:
-                self.tool_output("Detected dumb terminal, disabling fancy input and pretty output.")
+                print("Detected dumb terminal, disabling fancy input and pretty output.")
 
         self.file_watcher = file_watcher
         self.root = root
@@ -941,7 +936,7 @@ class InputOutput:
                 coder = self.get_coder()
 
                 if coder:
-                    await coder.commands.do_run("exit", "")
+                    await coder.commands.execute("exit", "")
                     return ""
                 else:
                     raise SystemExit
@@ -1039,14 +1034,22 @@ class InputOutput:
                 await output_task
             except (
                 asyncio.CancelledError,
-                Exception,
                 EOFError,
-                IndexError,
-                RuntimeError,
                 SystemExit,
                 SwitchCoderSignal,
             ):
                 pass
+            except (
+                Exception,
+                IndexError,
+                RuntimeError,
+            ):
+                import traceback
+
+                traceback_str = traceback.format_exc()
+                update_error_prefix(traceback_str)
+                update_error_prefix(str(output_task))
+                pass
 
     async def stop_task_streams(self):
         input_task = asyncio.create_task(self.stop_input_task())
@@ -1484,7 +1487,6 @@ class InputOutput:
 
     def assistant_output(self, message, pretty=None):
         if not message:
-            self.tool_warning("Empty response received from LLM. Check your provider account?")
             return
 
         show_resp = message
