@@ -24,9 +24,6 @@ from . import (
     load_skill,
     ls,
     remove_skill,
-    replace_all,
-    replace_line,
-    replace_lines,
     replace_text,
     show_numbered_context,
     thinking,
@@ -59,9 +56,6 @@ TOOL_MODULES = [
     load_skill,
     ls,
     remove_skill,
-    replace_all,
-    replace_line,
-    replace_lines,
     replace_text,
     show_numbered_context,
     thinking,
